#include <gtk/gtk.h>
#include <glade/glade.h>
#include "kf.h"
#include "preferences.h"

#define ISEMPTY(str) (str?str:"")

static void  kf_select_account             (GtkWidget *menuitem,
                                            gpointer user_data);
static void  kf_save_account               (KfPrefAccount *acc, GladeXML *glade);
static const gchar *value_of (GladeXML *glade, const gchar *name);

static void
on_text_data			            (GtkTreeViewColumn *tree_column,
                                             GtkCellRenderer *cell,
                                             GtkTreeModel *tree_model,
                                             GtkTreeIter *iter,
                                             gpointer data);
void        on_accounts_add_clicked        (GtkButton *button,
                                            gpointer user_data);
void        on_accounts_edit_clicked       (GtkButton *button,
                                            gpointer user_data);
void        on_accounts_rename_clicked     (GtkButton *button,
                                            gpointer user_data);
void        on_accounts_remove_clicked     (GtkButton *button,
                                            gpointer user_data);
static KfPrefAccount *get_selection (GtkWidget *widget);
static void hide (GladeXML *glade, const gchar *name);

void kf_accounts_connection_settings (void) {
	GladeXML *glade;
	GtkWidget *cs; /* Connection Settings window */
	GtkWidget *menu;
	GtkWidget *item;
	extern GList *kf_preferences_accounts;
	GList *account;
	KfPrefAccount *acc;
	GtkOptionMenu *optmenu;

	glade = glade_xml_new (kf_find_file ("kf.glade"), "connection_settings_window", NULL);
	glade_xml_signal_autoconnect (glade);

	optmenu = GTK_OPTION_MENU (glade_xml_get_widget (glade, "connection_konto_selector"));
	if (!optmenu)
		g_error ("Nie ma menu!\n");
	if (menu = gtk_option_menu_get_menu (optmenu)) {
		gtk_widget_destroy (menu);
	}

	menu = gtk_menu_new ();
	item = gtk_menu_item_new_with_label (_("[none]"));
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	g_object_set_data (G_OBJECT (item), "glade", glade);
	g_signal_connect (G_OBJECT (item), "activate",
			G_CALLBACK (kf_select_account), NULL);
	gtk_widget_show (item);
	for (account = kf_preferences_accounts; account; account = account->next) {
		acc = (KfPrefAccount *) account->data;

		item = gtk_menu_item_new_with_label (acc->name);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
		g_object_set_data (G_OBJECT (item), "glade", glade);
		g_signal_connect (G_OBJECT (item), "activate",
				G_CALLBACK (kf_select_account), acc);
		gtk_widget_show (item);
	}

	gtk_option_menu_set_menu (optmenu, GTK_WIDGET (menu));
	gtk_widget_show (menu);

	cs = glade_xml_get_widget (glade, "connection_settings_window");
	g_object_set_data_full (G_OBJECT (cs), "glade", glade, g_object_unref);
	kf_gui_show (cs);
}

static void  kf_select_account             (GtkWidget *menuitem,
                                            gpointer user_data) {
	KfPrefAccount *acc;
	GtkWidget *entry;
	GladeXML *glade = g_object_get_data (G_OBJECT (menuitem), "glade");
	g_assert (glade);
	entry = glade_xml_get_widget (glade, "connection_username");

	if (user_data) {
		acc = (KfPrefAccount *) user_data;
		gtk_entry_set_text (GTK_ENTRY (entry), ISEMPTY (acc->uname));
		entry = glade_xml_get_widget (glade, "connection_server");
		gtk_entry_set_text (GTK_ENTRY (entry), ISEMPTY (acc->server));
		entry = glade_xml_get_widget (glade, "connection_password");
		if (acc->save_password && acc->pass)
			gtk_entry_set_text (GTK_ENTRY (entry), ISEMPTY (acc->pass));
		else
			gtk_entry_set_text (GTK_ENTRY (entry), "");

		entry = glade_xml_get_widget (glade, "connection_save_password");
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (entry), acc->save_password);
		entry = glade_xml_get_widget (glade, "connection_port");
		if (acc->port > 0)
			gtk_spin_button_set_value (GTK_SPIN_BUTTON (entry), acc->port);
		else
			gtk_spin_button_set_value (GTK_SPIN_BUTTON (entry), acc->secure?5223:5222);
		entry = glade_xml_get_widget (glade, "connection_resource");
		gtk_entry_set_text (GTK_ENTRY (entry), ISEMPTY (acc->resource));
		entry = glade_xml_get_widget (glade, "connection_use_ssl");
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (entry), acc->secure);
	} else {
		/* Reste to defaults */
		gtk_entry_set_text (GTK_ENTRY (entry), "");
		entry = glade_xml_get_widget (glade, "connection_server");
		gtk_entry_set_text (GTK_ENTRY (entry), "");
		entry = glade_xml_get_widget (glade, "connection_password");
		gtk_entry_set_text (GTK_ENTRY (entry), "");
		entry = glade_xml_get_widget (glade, "connection_save_password");
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (entry), FALSE);
		entry = glade_xml_get_widget (glade, "connection_port");
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (entry), 5222);
		entry = glade_xml_get_widget (glade, "connection_resource");
		gtk_entry_set_text (GTK_ENTRY (entry), "kf");
		entry = glade_xml_get_widget (glade, "connection_use_ssl");
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (entry), FALSE);
	}

	g_object_set_data (G_OBJECT (gtk_widget_get_toplevel (GTK_WIDGET (entry))), "account", user_data);
}

void        on_connection_quit_clicked     (GtkButton *button,
                                            gpointer user_data)
{
	gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


void        on_connection_zapisz_clicked   (GtkButton *button,
                                            gpointer user_data)
{
	GladeXML *glade = glade_get_widget_tree (GTK_WIDGET (button));
	KfPrefAccount *acc;
	
	if (acc = g_object_get_data (G_OBJECT (gtk_widget_get_toplevel (GTK_WIDGET (button))), "account"))
		kf_save_account (acc, glade);
	
	gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


void        on_connection_ok_clicked       (GtkButton *button,
                                            gpointer user_data)
{
	GladeXML *iface = glade_get_widget_tree (GTK_WIDGET (button));
	GtkWidget *entry, *window;
	const gchar *server, *username, *password, *resource; 
	gint port;
	KfPrefAccount *acc;

	entry = glade_xml_get_widget (iface, "connection_server");
	server = gtk_entry_get_text (GTK_ENTRY (entry));
	entry = glade_xml_get_widget (iface, "connection_username");
	username = gtk_entry_get_text (GTK_ENTRY (entry));
	entry = glade_xml_get_widget (iface, "connection_password");
	password = gtk_entry_get_text (GTK_ENTRY (entry));
	entry = glade_xml_get_widget (iface, "connection_resource");
	resource = gtk_entry_get_text (GTK_ENTRY (entry));
	entry = glade_xml_get_widget (iface, "connection_port");
	port = gtk_spin_button_get_value (GTK_SPIN_BUTTON (entry));

	kf_jabber_set_connection (server, username, password, resource, port);
	kf_jabber_connect();

	if (acc = g_object_get_data (G_OBJECT (gtk_widget_get_toplevel (GTK_WIDGET (button))), "account")) {
		kf_save_account (acc, iface);
	}

	gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


/* Obs³uga "> Zaawansowane" w oknie ustawieñ po³±czenia */
gboolean    on_connection_eventbox_button_press_event (GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data)
{
	GladeXML *iface = glade_get_widget_tree (widget);
	GtkWidget *frame, *arrow;

	frame = glade_xml_get_widget (iface, "connection_frame");
	arrow = glade_xml_get_widget (iface, "arrow1");
	if (!GTK_WIDGET_VISIBLE (frame)) {
		gtk_widget_show (frame);
		gtk_arrow_set (GTK_ARROW (arrow), GTK_ARROW_DOWN, GTK_SHADOW_OUT);
	} else {
		GtkWidget *window;

		window = glade_xml_get_widget (iface, "connection_settings_window");
		gtk_widget_hide (frame);
		gtk_window_resize (GTK_WINDOW (window), 1, 1);
		gtk_arrow_set (GTK_ARROW (arrow), GTK_ARROW_RIGHT, GTK_SHADOW_OUT);
	}
}

/* Zamkniêcie okna ustawieñ po³±czenia */
gboolean    on_connection_settings_window_delete_event (GtkWidget *widget,
                                            GdkEvent *event,
                                            gpointer user_data)
{
	gtk_widget_destroy (widget);
	return TRUE;
}


static void  kf_save_account               (KfPrefAccount *acc, GladeXML *glade) {
	GtkWidget *save_password;
	GtkWidget *use_ssl;
	GtkWidget *port;

	save_password = glade_xml_get_widget (glade, "connection_save_password");
	use_ssl = glade_xml_get_widget (glade, "connection_use_ssl");
	port = glade_xml_get_widget (glade, "connection_port");
	
	g_free (acc->uname); acc->uname = g_strdup (value_of (glade, "connection_username"));
	g_free (acc->server); acc->server = g_strdup (value_of (glade, "connection_server"));
	g_free (acc->pass); 
	if (acc->save_password = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (save_password))) {
		acc->pass = g_strdup (value_of (glade, "connection_password"));
	} else {
		acc->pass = NULL;
	}

	acc->port = (gint) gtk_spin_button_get_value (GTK_SPIN_BUTTON (port));
	acc->secure = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (use_ssl));

	g_free (acc->resource); acc->resource = g_strdup (value_of (glade, "connection_resource"));
}

static const gchar *value_of (GladeXML *glade, const gchar *name) {
	GtkWidget *widget;

	widget = glade_xml_get_widget (glade, name);
	return gtk_entry_get_text (GTK_ENTRY (widget));
}


void        on_connection_dodaj_konto_clicked (GtkButton *button,
                                            gpointer user_data)
{
	GladeXML *glade = glade_get_widget_tree (GTK_WIDGET (button));
	GtkWidget *win, *parent;
	GtkWidget *label, *entry;
	gint response;

	parent = glade_xml_get_widget (glade, "connection_settings_window");
	win = gtk_dialog_new_with_buttons (_("Enter name..."), GTK_WINDOW (parent), GTK_DIALOG_DESTROY_WITH_PARENT,
                                                  GTK_STOCK_OK,
                                                  GTK_RESPONSE_ACCEPT,
                                                  GTK_STOCK_CANCEL,
                                                  GTK_RESPONSE_REJECT,
                                                  NULL);

	label = gtk_label_new (_("New account name:"));
	gtk_widget_show (label);
	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (win)->vbox), label);

	entry = gtk_entry_new ();
	gtk_entry_set_text (GTK_ENTRY (entry), _("Unnamed"));
	gtk_widget_show (entry);
	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (win)->vbox), entry);
	
	response = gtk_dialog_run (GTK_DIALOG (win));
	if (response != GTK_RESPONSE_ACCEPT) {
		gtk_widget_destroy (win);
	} else {
		KfPrefAccount *acc;
		const gchar *name;
		GtkWidget *item, *menu, *option;
		extern GList *kf_preferences_accounts;

		name = gtk_entry_get_text (GTK_ENTRY (entry));

		acc = kf_pref_account_new (name);

		option = glade_xml_get_widget (glade, "connection_konto_selector");
		menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (option));

		item = gtk_menu_item_new_with_label (acc->name);
		gtk_menu_shell_insert (GTK_MENU_SHELL (menu), item, 1);
		gtk_widget_show (item);
		g_object_set_data (G_OBJECT (item), "glade", glade);
		gtk_option_menu_set_history (GTK_OPTION_MENU (option), 1);
		g_signal_connect (G_OBJECT (item), "activate",
				G_CALLBACK (kf_select_account), acc);

		g_object_set_data (G_OBJECT (parent), "account", acc);
		kf_preferences_accounts = g_list_prepend (kf_preferences_accounts, acc);

		gtk_widget_destroy (win);
	}
}

void kf_accounts_editor (GladeXML *glade) {
	extern GList *kf_preferences_accounts;
	GList *tmp;

	GtkWidget *tv;
	GtkListStore *store;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	tv = glade_xml_get_widget (glade, "accounts_treeview");
	if (store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (tv)))) {
		gtk_list_store_clear (store);
	} else {
		store = gtk_list_store_new (1, G_TYPE_POINTER);

		renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Name",
                                                   renderer,
                                                   NULL);
		gtk_tree_view_column_set_cell_data_func (column,
				renderer,
				on_text_data,
				NULL, NULL);				
		gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);

		gtk_tree_view_set_model (GTK_TREE_VIEW (tv), GTK_TREE_MODEL (store));
	}

	for (tmp = kf_preferences_accounts; tmp; tmp = tmp->next) {
		KfPrefAccount *acc = tmp->data;
		GtkTreeIter iter;

		gtk_list_store_append (store, &iter);
		gtk_list_store_set (store, &iter,
                    0, acc,
                    -1);

	}
}

/* This function displays a text in a TreeView */
static void
on_text_data			            (GtkTreeViewColumn *tree_column,
                                             GtkCellRenderer *renderer,
                                             GtkTreeModel *tree_model,
                                             GtkTreeIter *iter,
                                             gpointer data)
{
	KfPrefAccount *acc;
	
	gtk_tree_model_get (tree_model, iter, 0, &acc, -1);

	if (acc) {
		g_object_set (G_OBJECT (renderer), "text", acc->name, NULL);
	}

}

void        on_accounts_add_clicked        (GtkButton *button,
                                            gpointer user_data) {
	GladeXML *glade = glade_get_widget_tree (GTK_WIDGET (button));
	GtkWidget *win, *parent;
	GtkWidget *label, *entry;
	gint response;

	parent = gtk_widget_get_toplevel (GTK_WIDGET (button)); 
	win = gtk_dialog_new_with_buttons (_("Enter name..."), GTK_WINDOW (parent), GTK_DIALOG_DESTROY_WITH_PARENT,
                                                  GTK_STOCK_OK,
                                                  GTK_RESPONSE_ACCEPT,
                                                  GTK_STOCK_CANCEL,
                                                  GTK_RESPONSE_REJECT,
                                                  NULL);

	label = gtk_label_new (_("New account name:"));
	gtk_widget_show (label);
	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (win)->vbox), label);

	entry = gtk_entry_new ();
	gtk_entry_set_text (GTK_ENTRY (entry), _("Unnamed"));
	gtk_widget_show (entry);
	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (win)->vbox), entry);
	
	response = gtk_dialog_run (GTK_DIALOG (win));
	if (response != GTK_RESPONSE_ACCEPT) {
		gtk_widget_destroy (win);
	} else {
		KfPrefAccount *acc;
		const gchar *name;
		GtkWidget *item, *menu, *option;
		extern GList *kf_preferences_accounts;

		name = gtk_entry_get_text (GTK_ENTRY (entry));

		acc = kf_pref_account_new (name);

		kf_preferences_accounts = g_list_prepend (kf_preferences_accounts, acc);

		kf_accounts_editor (glade); 
		gtk_widget_destroy (win);
	}
}

/* Opens a window with account options */
void        on_accounts_edit_clicked       (GtkButton *button,
                                            gpointer user_data) {
	KfPrefAccount *acc;
	if (acc = get_selection (GTK_WIDGET (button))) {
		GladeXML *glade;
		GtkWidget *cs; /* Connection Settings window */
		GtkWidget *button;
//		extern GList *kf_preferences_accounts;
//		GList *account;
//		GtkOptionMenu *optmenu;

		glade = glade_xml_new (kf_find_file ("kf.glade"), "connection_settings_window", NULL);
		glade_xml_signal_autoconnect (glade);

		hide (glade, "label5");
		hide (glade, "connection_konto_selector");
		hide (glade, "connection_dodaj_konto");
		hide (glade, "connection_ok");

		button = glade_xml_get_widget (glade, "connection_zapisz");
		gtk_widget_show (button);

/*	optmenu = GTK_OPTION_MENU (glade_xml_get_widget (glade, "connection_konto_selector"));
	if (!optmenu)
		g_error ("Nie ma menu!\n");
	if (menu = gtk_option_menu_get_menu (optmenu)) {
		gtk_widget_destroy (menu);
	}

	menu = gtk_menu_new ();
	item = gtk_menu_item_new_with_label (_("[none]"));
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	g_object_set_data (G_OBJECT (item), "glade", glade);
	g_signal_connect (G_OBJECT (item), "activate",
			G_CALLBACK (kf_select_account), NULL);
	gtk_widget_show (item);
	for (account = kf_preferences_accounts; account; account = account->next) {
		acc = (KfPrefAccount *) account->data;

		item = gtk_menu_item_new_with_label (acc->name);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
		g_object_set_data (G_OBJECT (item), "glade", glade);
		g_signal_connect (G_OBJECT (item), "activate",
				G_CALLBACK (kf_select_account), acc);
		gtk_widget_show (item);
	}

	gtk_option_menu_set_menu (optmenu, GTK_WIDGET (menu));
	gtk_widget_show (menu);*/

		cs = glade_xml_get_widget (glade, "connection_settings_window");
		g_object_set_data_full (G_OBJECT (cs), "glade", glade, g_object_unref);
		kf_select_account (cs, acc);
		kf_gui_show (cs);
	}
}

void        on_accounts_rename_clicked     (GtkButton *button,
                                            gpointer user_data) {
	KfPrefAccount *acc;
	if (acc = get_selection (GTK_WIDGET (button))) {
		GladeXML *glade = glade_get_widget_tree (GTK_WIDGET (button));
		GtkWidget *win, *parent;
		GtkWidget *label, *entry;
		gint response;

		parent = gtk_widget_get_toplevel (GTK_WIDGET (button)); 
		win = gtk_dialog_new_with_buttons (_("Enter name..."), GTK_WINDOW (parent), GTK_DIALOG_DESTROY_WITH_PARENT,
                                                  GTK_STOCK_OK,
                                                  GTK_RESPONSE_ACCEPT,
                                                  GTK_STOCK_CANCEL,
                                                  GTK_RESPONSE_REJECT,
                                                  NULL);

		label = gtk_label_new (_("New account name:"));
		gtk_widget_show (label);
		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (win)->vbox), label);

		entry = gtk_entry_new ();
		gtk_entry_set_text (GTK_ENTRY (entry), acc->name);
		gtk_widget_show (entry);
		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (win)->vbox), entry);
	
		response = gtk_dialog_run (GTK_DIALOG (win));
		if (response != GTK_RESPONSE_ACCEPT) {
			gtk_widget_destroy (win);
		} else {
			g_free (acc->name);
			acc->name = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));

			kf_accounts_editor (glade); 
			gtk_widget_destroy (win);
		}
	}
}
void        on_accounts_remove_clicked     (GtkButton *button,
                                            gpointer user_data) {
	GtkWidget *win;
	gint response;

	win = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (button))),
			GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_MESSAGE_QUESTION,
			GTK_BUTTONS_YES_NO,
			"Do you really want to remove this account?");
	response = gtk_dialog_run (GTK_DIALOG (win));
	gtk_widget_destroy (win);

	if (response == GTK_RESPONSE_YES) {
		KfPrefAccount *acc;

		acc = get_selection (GTK_WIDGET (button));
		if (acc) {
			extern GList *kf_preferences_accounts;

			kf_preferences_accounts = g_list_remove (kf_preferences_accounts, acc);
			kf_accounts_editor (glade_get_widget_tree (GTK_WIDGET (button))); 
		}
	}
}

static KfPrefAccount *get_selection (GtkWidget *widget) {
	GladeXML *glade = glade_get_widget_tree (widget);
	GtkWidget *tv;
	GtkListStore *store;
	GtkTreeSelection *sel;
	GtkTreeIter iter;
	
	tv = glade_xml_get_widget (glade, "accounts_treeview");
	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
	if (gtk_tree_selection_get_selected (sel, (GtkTreeModel **) &store, &iter)) {
		KfPrefAccount *acc;
		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, 0, &acc, -1);

		return acc;
	} else {
		return NULL;
	}

}

/* Hides a widget; internal */
static void hide (GladeXML *glade, const gchar *name) {
	GtkWidget *widget = glade_xml_get_widget (glade, name);
	gtk_widget_hide (widget);
}
