#include <gtk/gtk.h>
#include <stdio.h>

typedef struct {
	gchar *name;
	gchar *filename;
	GdkPixbuf *pixbuf;
} KfEmoticon;

void kf_emoticons_insert_text (GtkTextBuffer *buffer, const gchar *text);
GList *kf_emoticons_load (const gchar *filename);
KfEmoticon *kf_emoticon_new (const gchar *name, const gchar *path, const gchar *file);
void kf_emoticons_init (void);

GList *kf_emoticons_list = NULL;

/*
 * Insetrs text into a GtkTextBuffer buffer with emoticons
 * Since i have absolutely no idea how to do this, I have adapted
 * code from GnuGadu2 (http://gadu.gnu.pl/)
 * by Marcin Krzyanowski and Igor Popik.
 * Thanks a lot!
 */
void kf_emoticons_insert_text (GtkTextBuffer *buffer, const gchar *text) {
	GtkTextIter start_iter, end_iter;
	GtkTextMark *text_mark;
	GList *list;
	KfEmoticon *emot;

//	kf_emoticons_load ("emoticons.def");
	gtk_text_buffer_get_end_iter (buffer, &end_iter);
//	gtk_text_buffer_get_end_iter (buffer, &begin_of_inserted_text);
//	begin_of_inserted_text = end_iter;

	gtk_text_buffer_insert (buffer, &end_iter, text, -1);
	text_mark = gtk_text_buffer_get_insert (buffer);

	gtk_text_buffer_get_iter_at_mark (buffer, &start_iter, text_mark);
//	gtk_text_buffer_get_start_iter (buffer, &end_iter);
	gtk_text_buffer_get_end_iter (buffer, &end_iter);
//	gtk_text_iter_backward_char (&start_iter);

	for (list = kf_emoticons_list; list; list=list->next) {
		emot = list->data;
		while (gtk_text_iter_backward_search
				(&start_iter, emot->name, GTK_TEXT_SEARCH_VISIBLE_ONLY, &start_iter, &end_iter, NULL)) {

			gtk_text_buffer_delete (buffer, &start_iter, &end_iter);

			gtk_text_buffer_insert_pixbuf (buffer, &start_iter, emot->pixbuf);
			gtk_text_buffer_get_end_iter (buffer, &start_iter);
		}
	}
	
}

GList *kf_emoticons_load (const gchar *filename) {
	FILE *f;
	gchar line[1024];
	gchar *path;
	KfEmoticon *emot;

	path = g_path_get_dirname (filename);
	f = fopen (filename, "r");
	if (f == NULL)
		return kf_emoticons_list;
	
	while (fgets (line, 1024, f)) {
		gchar *e_name, *e_file, *tmp;
		e_name = line;

		for (tmp = line; *tmp != '\t' && *tmp != '\0' && *tmp != '\n'; tmp++)
			;
		if (*tmp == '\0' || *tmp == '\n')
			continue;
		else
			*tmp = '\0';
		
		e_file = ++tmp;

		for (; *tmp != '\0' && *tmp != '\n'; tmp++)
			;
		*tmp = '\0';

//		g_print ("Czytam emoticon %s (%s)... ", e_name, e_file);

		emot = kf_emoticon_new (e_name, path, e_file);
		if (emot) {
			kf_emoticons_list = g_list_append (kf_emoticons_list, emot);
//			g_print ("ok\n");
		}
	}
	fclose (f);
	
	g_free (path);

	return kf_emoticons_list;
}

/* Creates a new Emoticon object */
KfEmoticon *kf_emoticon_new (const gchar *name, const gchar *path, const gchar *file) {
	gchar *fullpath;
	KfEmoticon *emot;
	GdkPixbuf *pixbuf;

	fullpath = g_build_filename (path, file, NULL);
	/* FIXME: I don't know whether to share a single GdkPixbuf among all
	 * with the same filename isn't a better idea; however it would
	 * need a hash of GdkPixbufs indexed by their filenames, thus it
	 * would slow all engine down... So now every kind of emoticon
	 * has its own GdkPixbuf.
	 */
	pixbuf = gdk_pixbuf_new_from_file (fullpath, NULL);
	g_free (fullpath);
	if (pixbuf == NULL)
		return NULL;

	emot = g_new (KfEmoticon, 1);
	emot->name = g_strdup (name);
	emot->filename = g_strdup (file);
	emot->pixbuf = pixbuf;

	return emot;
}

void kf_emoticons_init (void) {
	GList *tmp;
	KfEmoticon *emot;
	
	kf_emoticons_list = kf_emoticons_load (kf_gui_find_file ("/emoticons/emoticons.def"));
	kf_emoticons_list = kf_emoticons_load (kf_config_file ("/emoticons/emoticons.def"));

	/* For debug purposes only */
//	for (tmp = kf_emoticons_list; tmp; tmp=tmp->next) {
//		emot = tmp->data;
//		g_print ("Zaladowano emoticon %s (%s)\n", emot->name, emot->filename);
//	}	
}
	
