#include <gtk/gtk.h>
#include <glade/glade.h>
#include "kf.h"
#include "gui.h"
#include "jabber.h"
#include "events.h"
#include "chat.h"
#include "message.h"

/* Event structure */
typedef struct {
	gint id;
	gchar *jid;
	gchar *fulljid;
	KfEventClass klass;
	gpointer data;
	GDestroyNotify notify;
} KfEventElement;

/* Notify window */
static GtkWidget *event_box = NULL;
/* ID of timeout, that hides notify window */
static gint timeout_id = 0;

static GtkWidget *event_box_create (void);
static gboolean on_event_box_button_press  (GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data);
static gboolean event_box_timeout (gpointer user_data);
static gchar *event_text (KfEventClass klass, gpointer user_data);

/* Event queue */
static KfEventElement *kf_event_element_new (const gchar *jid, KfEventClass klass);
static void kf_event_element_free (KfEventElement *event);
static void kf_event_element_release (KfEventElement *event);

static GSList *event_queue = NULL;


void kf_event (KfEventClass klass, gpointer user_data) {
	GtkWidget *label;
	gint screen_width, screen_height;
	gint window_width, window_height;
	gchar *text;
	
	if (event_box == NULL)
		event_box = event_box_create ();

	label = g_object_get_data (G_OBJECT (event_box), "label");

	text = event_text (klass, user_data);
	/* FIXME - it doesn't work properly!!! */
	gtk_window_resize (GTK_WINDOW (event_box), 1, 1);
//	gtk_widget_set_size_request (event_box, 0, 0);
	gtk_label_set_text (GTK_LABEL (label), text);
	g_free (text);	

	gtk_widget_show_all (event_box);
	
	/* Place Notify window */
	screen_width = gdk_screen_width ();
	screen_height = gdk_screen_height ();
	gtk_window_get_size (GTK_WINDOW (event_box), &window_width, &window_height);
	gtk_window_move (GTK_WINDOW (event_box), screen_width - window_width - 16,
			screen_height - window_height - 16);
//	g_print ("Notify window: %dx%d at (%d,%d)\n", window_width, window_height,
//			 screen_width - window_width - 16, screen_height - window_height - 16);

	/* Hide notify window in 10 seconds */
	if (timeout_id != 0)
		gtk_timeout_remove (timeout_id);
	timeout_id = gtk_timeout_add (10000, event_box_timeout, event_box);
}

static GtkWidget *event_box_create (void) {
	GtkWidget *window;
	GtkWidget *eventbox;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *image;
	GdkColor postie = {0, 0xFFFF, 0xFFFF, 0xC0C0};
	GdkColor black = {0, 0x0, 0x0, 0x0};

	window = gtk_window_new (GTK_WINDOW_POPUP);
	gtk_widget_modify_bg (window, GTK_STATE_NORMAL, &black);

	eventbox = gtk_event_box_new ();
	gtk_widget_show (eventbox);
	gtk_container_add (GTK_CONTAINER (window), eventbox);
	
	g_signal_connect (G_OBJECT (eventbox), "button-press-event",
			G_CALLBACK (on_event_box_button_press), NULL);
	gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &postie);

	hbox = gtk_hbox_new (FALSE, 6);
	gtk_container_add (GTK_CONTAINER (eventbox), hbox);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 8);

	image = gtk_image_new_from_file (kf_gui_find_file ("online.png"));
	gtk_widget_show (image);
	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);

	label = gtk_label_new ("blah");
	gtk_widget_show (label);
	gtk_container_set_border_width (GTK_CONTAINER (eventbox), 1);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

	g_object_set_data (G_OBJECT (window), "label", label);

	return window;
}

static gboolean on_event_box_button_press  (GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data) {
	gtk_widget_hide (gtk_widget_get_toplevel (widget));
}

static gboolean event_box_timeout (gpointer user_data) {
	gtk_widget_hide (user_data);
	return FALSE;
}

static gchar *event_text (KfEventClass klass, gpointer user_data) {
	gchar *text = NULL;
	if (klass == KF_EVENT_CLASS_MESSAGE) {
		KfJabberMessage *msg;
		KfJabberRosterItem *item;
		gchar *jid;

		msg = user_data;
		jid = g_strdup (msg->from);
		kf_jabber_jid_crop (jid);

		item = kf_jabber_roster_item_get (jid);
		text = g_strdup_printf (_("Message from %s"), item?item->display_name:msg->from);
		g_free (jid);

		kf_sound_play ("bass.wav");
	} else if (klass == KF_EVENT_CLASS_CHAT) {
		KfJabberMessage *msg;
		KfJabberRosterItem *item;
		gchar *jid;

		msg = user_data;
		jid = g_strdup (msg->from);
		kf_jabber_jid_crop (jid);

		item = kf_jabber_roster_item_get (jid);
		text = g_strdup_printf (_("Chat with %s"), item?item->display_name:msg->from);
		g_free (jid);
		kf_sound_play ("button.wav");
	} else if (klass == KF_EVENT_CLASS_PRESENCE) {
		KfJabberRosterItem *item;

		item = user_data;
		text = g_strdup_printf (_("%s is now %s"), item->display_name,
				_(kf_jabber_presence_type_to_string (item->presence)));
		kf_sound_play ("harp.wav");
	}
	return text;
}
/*
void kf_event_sound (const gchar *f) {
	gchar *cmd;

	cmd = g_strdup_printf ("play \"%s\" &", kf_find_file (f));
	system (cmd);
	g_free (cmd);
}*/

/* Kolejkowanie zdarzen */


/* Queues an event */
void kf_event_queue (KfEventClass klass, const gchar *jid, gpointer data, GDestroyNotify notify) {
	KfEventElement *event;

	event = kf_event_element_new (jid, klass);
	event->data = data;
	event->notify = notify;

	event_queue = g_slist_append (event_queue, event);
}

/* Release all events for jid */
void kf_event_release (const gchar *jid) {
	GSList *tmp;
	gchar *cropped_jid;
	cropped_jid = kf_jabber_jid_crop (g_strdup (jid));

	for (tmp = event_queue; tmp; tmp=tmp->next) {
		KfEventElement *event;

		event = tmp->data;

		if (strcmp (cropped_jid, event->jid) == 0) {
			kf_event_element_release (event);
			event_queue = g_slist_delete_link (event_queue, tmp);
			kf_event_element_free (event);
		}
	}

	g_free (cropped_jid);
}

void kf_event_release_all (void) {
	GSList *tmp;

	for (tmp = event_queue; tmp; tmp=tmp->next) {
		KfEventElement *event;

		event = tmp->data;

	//	if (strcmp (cropped_jid, event->jid) == 0) {
			kf_event_element_release (event);
			event_queue = g_slist_delete_link (event_queue, tmp);
			kf_event_element_free (event);
	//	}
	}
}

gboolean kf_event_get (const gchar *jid) {
	GSList *tmp;
	gchar *cropped_jid;
	cropped_jid = kf_jabber_jid_crop (g_strdup (jid));

	for (tmp = event_queue; tmp; tmp=tmp->next) {
		KfEventElement *event;

		event = tmp->data;

		if (strcmp (cropped_jid, event->jid) == 0) {
			g_free (cropped_jid);
			return TRUE;
		}
	}

	g_free (cropped_jid);
	return FALSE;
}

gboolean kf_event_get_all (void) {
	if (event_queue)
		return TRUE;
	else
		return FALSE;
}

static KfEventElement *kf_event_element_new (const gchar *jid, KfEventClass klass) {
	KfEventElement *event;

	event = g_new (KfEventElement, 1);
	event->jid = kf_jabber_jid_crop (g_strdup (jid));
	event->fulljid = g_strdup (jid);
	event->klass = klass;
	
	return event;
}
static void kf_event_element_free (KfEventElement *event) {
	g_free (event->jid);
	g_free (event->fulljid);
	g_free (event);
}

static void kf_event_element_release (KfEventElement *event) {
	if (event->klass == KF_EVENT_CLASS_MESSAGE) {
	} else if (event->klass == KF_EVENT_CLASS_CHAT) {
		kf_chat_window (event->fulljid);
	}
}

